<?php

defined( "ABSPATH" ) or die( "No script kiddies please!" );

class MonumetricAdsTxt {
    private $blogUuidCacheKey = "tbn_ad_settings";
    private $adsTxtCacheKey = "monu_adstxt";
    private $assetsLocation = "//monu.delivery/";
    private $adminSettingsKey = "monu_adstxt_config";

    public function __construct(){
        add_action( 'init', array( $this, 'adsTxtInit' ) );
        add_action( 'init', array( $this, 'setRewriteRule' ) );
        add_action( 'admin_init', array( $this, 'adminInit' ) );
        $this->get_wp_root = ABSPATH;
    }

    public function adsTxtInit() {
        add_rewrite_tag( '%adstxt%', 'true' );
        add_filter( 'redirect_canonical', array( $this, 'cancelAdsTxtRedirect' ) );
        add_filter( 'posts_pre_query', array( $this, 'disableQueryForAdsTxt' ), 10, 2 );
        add_filter( 'template_include', array( $this, 'loadAdsTxtTemplate' ) );
    }

    public function pluginsLoaded(){
        add_action( 'tbn_download_adstxt', array( $this, 'download') );
    }

    public function setRewriteRule($rules = NULL){
        if (!$rules) {
            add_rewrite_rule( '^ads\.txt$', 'index.php?adstxt=true', 'top' );
        } else {
            $rules['^ads\.txt$'] = 'index.php?adstxt=true';
            return $rules;
        }
    }

    public function setAndFlushRewriteRule(){
        $this->setRewriteRule();
        $option = get_option('rewrite_rules');
        if(is_array($option) && !isset($option['^ads\.txt$'])){
            global $wp_rewrite;
            $wp_rewrite->flush_rules( true );
        };
    }

    public function onActivationOrUpgrade(){
        if ( !wp_next_scheduled( 'tbn_download_adstxt' ) ) {
            wp_schedule_event( time(), 'hourly', 'tbn_download_adstxt' );
        }
        $this->createAdsTxtPostType();
        $this->setAndFlushRewriteRule();
        $options = get_option( $this->adminSettingsKey );
        $options = ($options && is_array($options)) ? $options : array();
        if ( !get_option( 'monu_adstxt_init' ) ) {
            update_option( $this->adminSettingsKey,  $options);
            update_option( 'monu_adstxt_init', 'true' );
        }
        $options['enable_redirect'] = true;
        update_option($this->adminSettingsKey, $options);
        return null;

    }

    public function onDeactivation(){
      $this->resetRedirectSettings();
      if(file_exists($this->get_wp_root . '/monu-renamed-ads.txt') && is_writable($this->get_wp_root . '/monu-renamed-ads.txt')){
        $this->renameAdsTxt("revert");
      }
        wp_clear_scheduled_hook( 'tbn_download_adstxt' );
    }

    public function uuidPath() {
      $uuid = $this->getUuid();
      if( $uuid ) {
        $exUUID = str_split($uuid);
        $uuidPath = array_shift($exUUID) . "/" . array_shift($exUUID) . "/" . implode($exUUID);
        return $uuidPath;
      }
      return null;
    }

    private function remoteAdsTxtPath() {
      $uuidPath = $this->uuidPath();
      if ($uuidPath) {
        return "http:" . $this->assetsLocation . "adstxt/". $uuidPath . ".txt";
      }
      return null;
    }

    //301 Redirect for Ads.txt
    public function addRedirect() {
      $remote = $this->remoteAdsTxtPath();
      if (is_admin()) {
        global $wp_rewrite;
        if($this->checkForLocalAdsTxt()){
          if($this->checkForWriteableAdsTxt()){
            $this->renameAdsTxt('rename', false);
          } else {
            $notWriteableMessage = "We are unable to rename your ads.txt file. In order to properly setup the Monumetric Hosted ads.txt there can not be a physical ads.txt file in your root directory. We recommend renaming it to something else (e.g. old-ads.txt).";
            add_action('admin_notices', $this->displayAdminNotice($notWriteableMessage, 'error')); 
            $this->resetRedirectSettings();
            return false;
          }
        }
        if (!isset($wp_rewrite->permalink_structure) || empty($wp_rewrite->permalink_structure)) {
          $permalinkMessage = 'The Monumetric hosted ads.txt requires that a permalink structure is set. The default (plain) WordPress permalink structure is not compatible with 301 redirects which are required to host your ads.txt on our server.<br>Please update the <a href="options-permalink.php" title="Permalinks">Permalink Structure</a>.';
          add_action('admin_notices', $this->displayAdminNotice($permalinkMessage, "error"));
          $this->resetRedirectSettings(); 
          return false;
        }
      }
      if ($remote !== NULL) {
        nocache_headers();
        $url = isset( $_SERVER['REQUEST_URI'] ) ? esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : '';
        if ( '/ads.txt' === $url ) {
          wp_redirect( $remote, 301, $x_redirect_by = "Monumetric" );
          exit();
        }
        return true;
      }
      return false;
      
    }

    public function displayAdminNotice($message, $type) {
      return function() use ($message, $type) {
        $class = "notice notice-". $type . " is-dismissible";
        ?>
        <div class="<?php echo $class; ?>">
          <p><?php echo "<b>Monumetric:</b>" . $message; ?></p>
        </div>
        <?php
      };
    }

    public function verifyAdstxtRedirect() {
      global $wp;
      $options = get_option($this->adminSettingsKey);
      // Create request
      $response = wp_remote_get(home_url($wp->request) . "/ads.txt", array(
          'timeout' => 5,
          'headers' => array('Cache-Control' => 'no-cache'),
      ));
      if (
          !is_wp_error($response)
          && isset($response['http_response'])
          && $response['http_response'] instanceof \WP_HTTP_Requests_Response
          && method_exists($response['http_response'], 'get_response_object')
      ) {
          $location_url = $response['http_response']->get_response_object()->url;
          $target_host = $newString = str_replace('/', '', $this->assetsLocation);
          $url_parse = wp_parse_url($location_url);
          if ($url_parse['host'] == $target_host) {
              $options['confirm_redirect'] = true;
              update_option($this->adminSettingsKey, $options);
              return true;
          } else {
              add_action('admin_notices', $this->displayAdminNotice('The ads.txt is not redirecting to the correct location. Please remove any existing redirects to your ads.txt and clear any cache from both WordPress and your Server.', 'error'));
              $options['confirm_redirect'] = false;
              update_option($this->adminSettingsKey, $options);
              return false;
          }
      } 
      add_action('admin_notices', $this->displayAdminNotice("Unable to verify your ads.txt redirection.", "error"));
      $this->resetRedirectSettings();
      return false;
    }

    public function resetRedirectSettings() {
      $options = get_option($this->adminSettingsKey);
      if ($options) {
        $options['enable_redirect'] = false;
        $options['confirm_redirect'] = false;
        $updated = update_option($this->adminSettingsKey, $options);
      }
    }

    public function cancelAdsTxtRedirect( $redirect ) {
        if ( get_query_var( 'adstxt' ) ) {
            return false;
        }
        return $redirect;
    }

    public function loadAdsTxtTemplate( $template ) {
        if ( get_query_var( 'adstxt' ) === 'true' ) {
            header("Content-Type: text/plain");
            header("Content-Type-Version: " . "7.4.3");
            $template = dirname( __FILE__ ) . '/templates/ads.php';
        }
        return $template;
    }

    public function disableQueryForAdsTxt( $posts, $query ) {
        if ( $query->is_main_query() && isset($query->query_vars['adstxt']) && 'true' === $query->query_vars['adstxt'] ) {
            $posts = array();
        }
        return $posts;
    }

    public function createAdsTxtPostType() {
        register_post_type('monuAdsTxt', array(
            'labels'       => array(
                'name'          => __( 'ads.txt' ),
                'singular_name' => __( 'ads.txt' ),
            ),
            'public'       => false,
            'has_archive'  => false,
            'rewrite'      => false,
            'show_ui'      => true,  // TODO: should probably have some sort of custom UI
            'show_in_menu' => false, // Since we're manually adding a Sitemaps menu, no need to auto-add one through the CPT.
            'supports'     => array(
                'title',
            ),
        ));

        global $wpdb;
        $sitemap_id = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_type = %s AND post_name = %s LIMIT 1", 'monuAdsTxt', 'ads.txt' ) );
        if ( $sitemap_id ) {
            $sitemap_data = array(
                'post_name' => 'ads.txt',
                'post_title' => 'ads.txt',
                'post_type' => 'monuAdsTxt',
                'post_status' => 'publish'
            );
            $sitemap_id = wp_insert_post( $sitemap_data );
        }
    }

    private function getUuid() {
        $options = get_option( $this->blogUuidCacheKey );
        if ( isset($options['uuid']) ){
            return $options['uuid'];
        }
        return null;
    }

    public function download( $uuid = null ) {
        $uuid = $uuid ? $uuid : $this->getUuid();
        $exUUID = str_split($uuid);
        $uuidPath = array_shift($exUUID) . "/" . array_shift($exUUID) . "/" . implode($exUUID);
        $response = wp_remote_get( "http:" . $this->assetsLocation . "adstxt/". $uuidPath . ".txt" );
        if( !is_wp_error($response) && $response['response']['code'] == 200 ) {

            $oldAdsTxt = get_option( 'monu_adstxt' );
            if ( !$oldAdsTxt || ($oldAdsTxt && $oldAdsTxt!= $response['body']) ) {
                update_option( $this->adsTxtCacheKey, $response['body'] );

                if (function_exists('ccfm_clear_cache_for_me')) {
                    ccfm_clear_cache_for_me( 'mmt_adstxt' );
                }
            }

            return $response['body'];
        }
        return null;
    }

    public function adminSettingsValidate( $args ){
        $this->setAndFlushRewriteRule();
        add_settings_error( $this->adminSettingsKey, 'invalid_uuid', "Saved successfully.", $type = 'updated' );
        return $args;
    }

    public function adminInit(){
        register_setting( $this->adminSettingsKey, $this->adminSettingsKey, array( $this, "adminSettingsValidate" ) );
    }

    public function isClearCacheforMePluginSupportedCacheExists() {
        return function_exists( 'w3tc_pgcache_flush' )
            || function_exists( 'wp_cache_clean_cache' )
            || class_exists( 'WpeCommon' )
            || method_exists( 'WpFastestCache', 'deleteCache' );
    }

    public function isClearCacheforMePluginInstalled() {
        $allPlugins = get_plugins();
        return isset($allPlugins['clear-cache-for-widgets/clear-cache-for-widgets.php']) && isset($allPlugins['clear-cache-for-widgets/clear-cache-for-widgets.php']['Name']);
    }

    public function isClearCacheforMePluginActivated() {
        return function_exists('is_plugin_active') && is_plugin_active( 'clear-cache-for-widgets/clear-cache-for-widgets.php' );
    }

    public function checkForLocalAdsTxt() {
      if(file_exists($this->get_wp_root . '/ads.txt')) {
        return true;
      }
      return false;
    }

    public function conflicts() {
      $fileLocation = plugin_dir_path(__FILE__) . "AdminNotices/adstxt.json";
      $conflictsArray;
      if($fileLocation){
        $contents = file_get_contents($fileLocation);
        $conflictsArray = json_decode($contents, true);
      } else {
        return null;
      }
      return $conflictsArray;
    }

    public function checkForWriteableAdsTxt() {
      if(file_exists($this->get_wp_root . '/ads.txt') && is_writable($this->get_wp_root . '/ads.txt')) {
        return true;
      } else if (file_exists($this->get_wp_root . '/ads.txt') && !is_writable($this->get_wp_root . '/ads.txt')){
        return false;
      }
    }

    public function renameAdsTxt($string, $refreshOnChange = true) {
      $original = "/ads.txt";
      $renamed = "monu-renamed-ads.txt";
      if($string === "rename") {
        rename($this->get_wp_root . $original, $this->get_wp_root . $renamed);
      } else if($string === "revert"){
        rename($this->get_wp_root . $renamed, $this->get_wp_root . $original);
      }
      if ($refreshOnChange) {
        echo '<META HTTP-EQUIV=Refresh CONTENT="0; URL=' . $_SERVER['REQUEST_URI'] . '">';
      }
      return;
    }

    private function customLinesTemplate($options, $customRecords) {
      ?> 
         <br/>
         <div class="monu-label">
           <label for="<?php echo $this->adminSettingsKey; ?>[custom_records]">Non-Monumetric Ads.txt Lines <i><?php echo (isset($options['enable_redirect']) && ($options['enable_redirect'] !== false)) ? '(disabled)' : null; ?></i></label>
         </div>
         <textarea
             class="large-text"
             name="<?php echo $this->adminSettingsKey; ?>[custom_records]"
             id="<?php echo $this->adminSettingsKey; ?>[custom_records]"
             placeholder="pub-111122223333444455, RESELLER, a1b2c3d4e5f67890"
             
             ><?php echo esc_textarea($customRecords)?></textarea>
         <br/><br/>
     <?php
     return null;
   }

   private function thirdPartyLinesMessage($message) {
     $messages = array(
       'cpt_add_lines' => 'If you run any lines outside of Monumetric, please post those in the "Non-Monumetric Ads.txt Lines" box below:',
       'redirect_add_lines' => 'If you run any lines outside of Monumetric, please post those within the <a href="https://console.monumetric.com/" target="_blank">Monumetric Console</a> "Settings" section under the "Ads.txt Settings" tab.',
     );
     return $messages[$message];
   }

   public function renderManager() {
     $options = get_option( $this->adminSettingsKey );
     $redirectStatus = isset($options['enable_redirect']) && $options['enable_redirect'] == true;
       ?>
       <div class="monu-panel-detail">
         <h3>What is Ads.txt?</h3>
         <p class="mt-0">Ads.txt is a file that creates a secure method for publishers and advertisers to declare who is authorized to buy ad inventory on your site while making it harder for bad actors to profit from selling counterfeit inventory. This allows brands to have confidence that they are competing to buy ads against other verified advertisers. <a href="https://iabtechlab.com/ads-txt/" target="_blank">Read More.</a></p>
         <p>This file must be public record in your root directory. Unless otherwise informed through notifications in your Wordpress dashboard, the Monumetric plugin will manage this record for you, ensuring all possible advertisers are able to bid on your site to optimize revenue.</p>
         <p><?php echo (!$redirectStatus) ? $this->thirdPartyLinesMessage('cpt_add_lines') : null;?></p> 
       <?php   
       if (!$redirectStatus) {
         if ( $this->isClearCacheforMePluginSupportedCacheExists() )
             if ( !$this->isClearCacheforMePluginInstalled() ) {
                 echo '<p class="monu-notice updated notice notice-error" style="padding:11px 15px">You have caching enabled on your site, which can cause problems with keeping your ads.txt up-to-date. If you install the \'<a href="https://wordpress.org/plugins/clear-cache-for-widgets/">Clear Cache for Me</a>\' plugin, then we can automatically clear your site\'s cache each time there is an update.</p>';
             } else if ( !$this->isClearCacheforMePluginActivated() ){
                 echo '<p class="monu-notice updated notice notice-error" style="padding:11px 15px">You have \'<a href="https://wordpress.org/plugins/clear-cache-for-widgets/">Clear Cache for Me</a>\' installed, but not activated, Please activate this plugin so that we can automatically clear your site\'s cache each time there is an update to your ads.txt</p>';
             }
         ?>
         <hr>
         <?php
         $warnings = $this->conflicts();
         foreach($warnings['plugins'] as $k => $v){
           if(is_plugin_active($v['path'])){
             ?>
             <section class="monu-notice notice notice-error is-dismissible">
               <p>We have detected the <strong> <?php echo $v['name']; ?> </strong> plugin is active and conflicting with Monumetric's ability to keep your ads.txt current. <br>There cannot be two different sources of an Ads.txt file on your site. If the conflict persists, your revenue and site reputation will be negatively affected.
               </p>
               <ol>
                 <li>Please combine the information within <strong> <?php echo $v['name']; ?> </strong> into the 'Non-Monumetric Ads.txt Lines' box under the Ads.txt tab in the Monumetric plugin</li>
                 <li>Disable the <strong> <?php echo $v['name']; ?> </strong> plugin</li>
                 <li>Email support@monumetric.com with any questions</li>
               </ol>
             </section>
           <?php
           }
         }
         ?>
         <!-- Physical ads.txt dection and notification for Monumetric Plugin Dashboard -->
         <?php
         if($this->checkForLocalAdsTxt() && $this->checkForWriteableAdsTxt()) {
           ?>
           <form method="post" class="monu-notice notice notice-error is-dismissible"  id="rename-adstxt-form" style="padding: 10px;">
             <p>We have detected a physical copy of ads.txt on your server. Your Ads.txt is managed by Monumetric. To avoid conflicts and losses in revenue, please remove the ads.txt file from your server. If you run any lines outside of Monumetric, please post those in the "Non-Monumetric Ads.txt Lines" box below</a>.</p>
             <p>If you prefer to use an ads.txt file on your server, Monumetric can still automatically update our lines if FTP credentials are securely saved within your Monumetric Console 'Settings' section under the 'Ads.txt Settings' tab. This automation is not able to edit other information within the Ads.txt file except for lines specific to Monumetric. <a href="https://accounts.monumetric.com/" target="_blank">Monumetric Account Login</a> </p>
             <p>Clicking the button below will rename your ads.txt file. This will allow Monumetric to manage your Ads.txt. For back-up purposes, the file will be renamed to "monu-renamed-ads.txt". Be sure to add any additional lines to the "Non-Monumetric Ads.txt Lines" section of below.</p>
           <?php
           submit_button('Rename File', 'monu-button', 'rename-adstxt', false);
           if(isset($_POST['rename-adstxt'])) {
             $this->renameAdsTxt("rename");
           }
           ?>
           </form>
           <?php
         } else if($this->checkForLocalAdsTxt() && !$this->checkForWriteableAdsTxt())
           // If content is not rewriteable just display the message
         {
           ?>
           <section class="monu-notice notice notice-error">
             <p>We have detected a physical copy of ads.txt on your server. Your Ads.txt is managed by Monumetric. To avoid conflicts and losses in revenue, please remove the Ads.txt file from your server. If you run any lines outside of Monumetric, please post those in the "Non-Monumetric Ads.txt Lines" box below</a>.</p>
             <p>If you prefer to use an ads.txt file on your server, Monumetric can still automatically update our lines if FTP credentials are securely saved within your Monumetric Console 'Settings' section under the 'Ads.txt Settings' tab. This automation is not able to edit other information within the Ads.txt file except for lines specific to Monumetric. <a href="https://accounts.monumetric.com/" target="_blank">Monumetric Account Login</a> </p>
           </section>
           <?php
         }
         ?>
         <?php
         // Option to revert back to physical ads.txt if something with our plugin isn't functioning
         if(file_exists($this->get_wp_root . '/monu-renamed-ads.txt') && is_writable($this->get_wp_root . '/monu-renamed-ads.txt') && !$this->checkForLocalAdsTxt())
           {
           ?>
           <form method="post" class="monu-notice notice notice-success is-dismissible"  id="revert-adstxt-form" style="padding: 10px;">
             <p>Thank you for renaming your ads.txt, if you are experiencing difficulties after renaming your ads.txt, you can revert back to a physical copy by clicking the 'Revert Ads.txt' button below. </p>
           <?php
           submit_button('Revert Ads.txt', 'monu-button', 'revert-adstxt', false);
           if(isset($_POST['revert-adstxt'])) {
             $this->renameAdsTxt("revert");
           }
           ?>
           </form>
           <?php
         }
       }
         ?>
         <form method="post" action="options.php">
             <?php
                 settings_fields( $this->adminSettingsKey );
                 do_settings_sections( __FILE__ );
                 $customRecords = ( isset( $options['custom_records'] ) && $options['custom_records'] != '' ) ? $options['custom_records'] : '';
                 $redirectAds_txt = ( isset( $options['enable_redirect'] ) && !$options['enable_redirect']) ? $options['enable_redirect'] : false;
             ?>

             <div class="monu-adline">
               <?php 
                 if (!$redirectStatus) {
                   echo $this->customLinesTemplate($options, $customRecords);
                 } else {
                   ?> 
                   <input type="hidden" name="<?php echo $this->adminSettingsKey; ?>[custom_records]" value="<?php echo esc_textarea($customRecords)?>">
                   <?php
                 }
               ?>
                 <h3>Monumetric Hosted Ads.txt</h3>
                 <p>This is the default settings for maintaining your ads.txt, this allows us to keep your ads.txt up to date without the worry of caching or confilcts. Initially, this may require that you disable any existing ads.txt plugins and clear any cache (WordPress and Server).</p>
                 <p><?php echo ($redirectStatus) ? $this->thirdPartyLinesMessage('redirect_add_lines') : null;?></p>
                 <input type="checkbox"
                   class="redirect-selection"
                   id="<?php echo $this->adminSettingsKey; ?>[custom_records]"
                   name="<?php echo $this->adminSettingsKey; ?>[enable_redirect]"
                   <?php echo (isset($options['enable_redirect']) && ($options['enable_redirect'] !== false)) ? 'checked' : ''; ?>>
                   Enable Monumetric Hosted Ads.txt<i> - beta</i>
                 </input>
                 <p><i>* If there is a physical ads.txt file it will be renamed to monu-renamed-ads.txt.</i><p>
                 <?php submit_button( 'Save Changes', 'monu-button', 'monu_adstxt_save', false ); ?>
               </div>
             <div class="clearfix"></div>
         </form>
     </div>
       <?php
   }
}