<?php

defined( "ABSPATH" ) or die( "No script kiddies please!" );
require( $basePath . "/libs/adstxt.php" );

// the following is for older php version
// becase create_function cannot call a function in an object!
function monumetricAdWidgetRender( $args ){
    printf( $args['before_widget'] );
    $tbnid = explode( '-', $args['widget_id'], 2 );
    printf("<!--MONUMETRIC " . trim($tbnid[1]) . " STARTS-->" .
           "<div id=\"mmt-" . $tbnid[1] . "\"></div>" .
           "<script type=\"text/javascript\" data-cfasync=\"false\">" .
           '$MMT = window.$MMT || {}; $MMT.cmd = $MMT.cmd || [];' .
           '$MMT.cmd.push(function(){ $MMT.display.slots.push(["' . $tbnid[1] . '"]); })' .
           "</script>" .
           "<!--MONUMETRIC " . trim($tbnid[1]) . " ENDS-->");
    printf( $args['after_widget'] );
}

class MonumetricAdvertisement {
    private $adminSettingsKey = "tbn_ad_settings";
    private $blogUuidCacheKey = "tbn_ad_settings";
    private $adDetailsCacheKey = "tbn_ad_provider_code";
    private $adLocation = "//monu.delivery/";
    private $isModern = false;

    public function __construct(){
        $this->isModern = ( version_compare( PHP_VERSION, '5.3.0' ) >= 0 ) && !( extension_loaded( "apc" ) && ini_get( "apc.enabled" ) );
        add_action( 'admin_init', array( $this, 'adminInit' ) );
        // add_action( 'wp_head', array( $this, 'echoAdScript' ) );
        add_action('wp_head', array($this, 'loadDeliveryScript') );
        add_action( 'widgets_init', array( $this, 'registerWidgets' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'adminEnqueueScripts') );
        add_action( 'tbn_download_ad_details', array( $this, 'downloadAdDetails') );
    }

    /**
     * Load delivery script based off exclusions settings
     * Loads delivery script in the body or the head depending on the TM3 configuration
     */
    public function loadDeliveryScript() {
        $options = get_option( $this->blogUuidCacheKey );
        $exclusions = ( isset( $options['exclusions']['user-roles'] ) && $options['exclusions']['user-roles'] != '' ) ? $options['exclusions']['user-roles'] : '';
        $adDetails = $this->getAdDetails();
        if(function_exists('wp_get_current_user')){
          $current_user = wp_get_current_user();
          $current_role = (array) $current_user->roles;
          if($exclusions && in_array($current_role[0], $exclusions)){
            return;
          }
        }
        if($adDetails && $adDetails->loadInBody) {
            // The number at the end sets the priority, the higher it is, the lower in the footer it will be loaded.
            return add_action( 'wp_footer', array( $this, 'echoAdScript' ), 20 );
        } else {
            return $this->echoAdScript();
        }
        
    }

    public function onActivationOrUpgrade(){
        if ( !wp_next_scheduled( 'tbn_download_ad_details' ) ) {
            wp_schedule_event( time(), 'hourly', 'tbn_download_ad_details' );
        }
    }

    public function onDeactivation(){
        wp_clear_scheduled_hook( 'tbn_download_ad_details' );
    }

    private function getUuid(){
        $options = get_option( $this->blogUuidCacheKey );
        if ( isset($options['uuid'])){
            //Trim to account for when user adds an extra leading/trailing space to the UUID
            return trim($options['uuid']);
        }
        return null;
    }

    public function adminEnqueueScripts(){
        wp_enqueue_script( 'jquery-ui-dialog' );
        wp_enqueue_script( 'monumetric-manager', plugins_url( '/../js/monumetric-manager.js', __FILE__ ), array(), '0.4', true );
    }

    public function downloadAdDetails( $uuid = null ){
        if( !$uuid ){
            $uuid = $this->getUuid();
        }
        if( !$uuid ){
            return null;
        }

        $exUUID = str_split($uuid);
        $uuidPath = array_shift($exUUID) . "/" . array_shift($exUUID) . "/" . implode($exUUID);

        $response = wp_remote_get( "http:" . $this->adLocation . "site/". $uuidPath . ".js" );

        if( !is_wp_error($response) && $response['response']['code'] == 200 ) {
            update_option( $this->adDetailsCacheKey, $response['body'] );
            $adDetails = $this->getAdDetails($response['body']);
            return $response['body'];
        }
        return null;
    }

    public function getConfigScript(){
        $adDetails = get_option( $this->adDetailsCacheKey );
        if( $adDetails ) {
            return $adDetails;
        }
        return null;
    }

    public function getAdDetails( $script = null ){
        $adDetails = $script ? $script : $this->getConfigScript();
        if( $adDetails ) {
            $mmtScript = preg_split('/$\R?^/m', $adDetails);
            $mmtConfiguration = explode("\$MMT.configuration =", rtrim(trim($mmtScript[0]), ";"));
            $jsonContent = trim($mmtConfiguration[1]);
            return json_decode($jsonContent);
        }
        return null;
    }

    public function hasAdDetails(){
        return $this->getConfigScript() ? true : false;
    }

    public function echoAdScript(){
        $uuid = $this->getUuid();
        $adDetails = $this->getAdDetails();
        if( $uuid ){
            $exUUID = str_split($uuid);
            $async = ($adDetails->loadAsync == 1) ? "async='true'" : '';
            $defer = ($adDetails->loadDefer == 1) ? " defer" : '';
            $uuidPath = array_shift($exUUID) . "/" . array_shift($exUUID) . "/" . implode($exUUID);
            echo "\n<script data-cfasync='false' " . $async . $defer . " src='" . $this->adLocation . "site/". $uuidPath . ".js" . "' type='text/javascript'></script>\n";
        }
    }

    public function registerWidgets(){
        $adDetails = $this->getAdDetails();
        if( !$adDetails ){
            return null;
        }
        $deviceType = array('desktop', 'tablet', 'mobile');
        foreach( $adDetails->product->displayAds->slots as $slot ) {
            $label = '[' . $slot->placement . '|Pos ' . $slot->pos . '] ';
            foreach( $deviceType as $device ) {
                if ( !isset($slot->sizeConfiguration->$device)) {
                    continue;
                }
                $sizes = $slot->sizeConfiguration->$device;
                if(sizeof($sizes) > 0){
                    $label = $label . strtoupper($device[0]) . ':';
                    foreach( $sizes as $size ) {
                        $label = $label . $size[0] . 'x' . $size[1] . ', ';
                    }
                    $label = rtrim($label, ",") . ' ';
                }
            }
            if ( !$this->isModern ) {
                include( dirname( __FILE__ ) . '/older-functions.php' );
                continue;
            }
            include( dirname( __FILE__ ) . '/modern-functions.php' );
        }
    }

    public function setRewriteRule(){
        add_rewrite_rule( '^ads\.txt$', 'index.php?adstxt=true', 'top' );
        $option = get_option('rewrite_rules');
        if(is_array($option) && !isset($option['^ads\.txt$'])){
            global $wp_rewrite;
            $wp_rewrite->flush_rules( true );
        };
    }

    public function adminSettingsValidate( $args ){
        if( !isset( $args['uuid'] ) || trim( $args['uuid'] ) == "" ) {
            $args['uuid'] = '';
            add_settings_error( $this->adminSettingsKey, 'invalid_uuid', 'Please enter a valid advertisement id!', $type = 'error');
            return $args;
        } else if ( is_null( $this->downloadAdDetails( trim( $args['uuid'] ) ) ) ) {
            $args['uuid'] = '';
            add_settings_error( $this->adminSettingsKey, 'invalid_uuid', 'Could not fetch advertisement for that id, please retry!', $type = 'error');
            return $args;
        }
        $message = filter_input(INPUT_POST, 'refresh-ad') ? "Successfully refreshed ads." : "Saved successfully.";
        $monuAdsTxt = new MonumetricAdsTxt();
        $monuAdsTxt->download(trim( $args['uuid'] ));
        add_settings_error( $this->adminSettingsKey, 'invalid_uuid', $message, $type = 'updated' );
        $this->setRewriteRule();
        return $args;
    }

    public function adminInit(){
        register_setting( $this->adminSettingsKey, $this->adminSettingsKey, array( $this, "adminSettingsValidate" ) );
    }

    public function renderManager(){
        $adDetails = $this->getAdDetails(); 
        global $wp_roles;
        $userRoles = $wp_roles->roles;
        ?>

        <div class="monu-panel-detail">
            <form method="post" action="options.php" class="default-form">
                <?php
                    settings_fields( $this->adminSettingsKey );
                    do_settings_sections( __FILE__ );
                    $options = get_option( $this->adminSettingsKey );
                    $uuid = ( isset( $options['uuid'] ) && $options['uuid'] != '' ) ? $options['uuid'] : '';
                    $exclusions = ( isset( $options['exclusions']['user-roles'] ) && count($options['exclusions']['user-roles']) != 0 ) ? $options['exclusions']['user-roles'] : array();
                ?>
                    <div class="monu-label">
                        <label for="tbn-ad-id">Monumetric Advertisement ID</label>
                    </div>
                    <input required name="<?php echo $this->adminSettingsKey; ?>[uuid]" id="tbn-ad-id" type="text" class="regular-text" placeholder="Example: a12bcdef-34ab-4578-98c7-d65ef43210a1" value="<?php echo $uuid ; ?>">
                    <div class="monu-tip">
                        Do not edit this number unless instructed to by Monumetric.
                    </div>
                    <div class="monu-exclusions">
                      <div class="monu-label">
                        Role Exclusions
                      </div>
                      <div class="monu-current-exclusions"></div>
                      <div class="monu-exclusion-dropdown-parent">
                        <div class="monu-exclusion-dropdown">
                          <div class="monu-selected">
                            User roles
                          </div>
                          <div class="monu-chevron">
                            <span class="dashicons dashicons-arrow-down-alt2"></span>
                          </div>
                        </div>
                        <div class="monu-exclusion-options">
                          <?php
                            foreach($userRoles as $role => $details){
                              ?>
                              <div class="monu-option">
                                <input name="<?php echo $this->adminSettingsKey; ?>[exclusions][user-roles][]" class="monu-user-role-exclusions" type="checkbox" value="<?php echo strtolower($details['name'])?>" <?php echo in_array( strtolower($details['name']), $exclusions) ? 'checked' : '' ?> >
                                <label for="<?php echo $details['name']; ?>"><?php echo $details['name']; ?></label>
                              </div>
                              <?php
                            }
                          ?>
                        </div>
                      </div>
                      <p class="monu-tip">Select which roles to exclude ads from</p>
                    </div>
                    <div class="monu-support-notification">
                    <?php
                        if($uuid) { ?>
                        <p>
                            If there have been changes to your ads with Monumetric they will automatically update in about an hour.
                            If you want to fetch those changes manually, please click the "Refresh Ads" button below.
                        </p>
                    <?php } ?>
                    </div>

                    <?php
                        submit_button( 'Save Changes', 'monu-button', 'save-id', false );
                        if( $uuid && $uuid != "") {
                            echo "&nbsp; &nbsp;";
                            submit_button( 'Refresh Ad', 'monu-button', 'refresh-ad', false );
                        }
                    ?>
            </form>
                <div class="monu-support-notification">

                    <p>
                    Click here to email Monumetric support: <a href="mailto:support@monumetric.com">support@monumetric.com</a>
                    </p>
                    <p>
                    Ads with Monumetric: <?php echo (isset($adDetails->product->displayAds->slots)) ? sizeOf($adDetails->product->displayAds->slots) : "0"; ?>
                    </p>
                </div>
            </div>
         <?php
    }
}

?>
