<?php

defined( "ABSPATH" ) or die( "No script kiddies please!" );

class AMPAds {
    private $adminSettingsKey = "monu_ampads_config";
    private $adDetailsCacheKey = "tbn_ad_provider_code";

    public function __construct(){
        add_action( 'admin_init', array( $this, 'adminInit' ) );
        add_action( 'template_redirect', array( $this, 'setAMPTemplateFilter' ));
    }

    public function getAdConfig(){
        $adDetails = get_option( $this->adDetailsCacheKey );
        if( $adDetails != false ) {
            $mmtScript = preg_split('/$\R?^/m', $adDetails);
            $mmtConfiguration = explode("\$MMT.configuration =", rtrim(trim($mmtScript[0]), ";"));
            $jsonContent = trim($mmtConfiguration[1]);
            return json_decode($jsonContent);
        }
        return false;
    }

    public function setAMPTemplateFilter() {
        if ( $this->isAMPAdsEnabled() && ( ( function_exists('is_amp_endpoint') && is_amp_endpoint() ) || ( function_exists('ampforwp_is_amp_endpoint') && ampforwp_is_amp_endpoint() ) ) && is_single() ) {
            add_filter( 'the_content', array($this, 'insertAMPAds' ));
        }
    }

    public function hasAMPAd() {
        $adConfig = $this->getAdConfig();
        if ( $adConfig &&
             $adConfig->product->amp &&
             $adConfig->product->amp->enabled &&
             $adConfig->product->amp->configuration &&
             $adConfig->product->amp->configuration->mobile ) {
            return $adConfig;
        }
        return false;
    }

    public function getAMPAd() {
        $adConfig = $this->hasAMPAd();
        if (!$adConfig ) {
            return false;
        }
        $slot = "/" . $adConfig->dfp->id . "/" . $adConfig->adUnit;
        $bids = (array) $adConfig->product->amp->configuration->mobile->prebidBidders;

        $ampPrebidConfig = array(
            "prebid"=>array(
                "requestBidsDuration" => 1000,
                "adUnits" => array(
                    array(
                        "code" => $slot,
                        "sizes" => array(array(300, 250)),
                        "bids" => $bids
                    )
                )
            )
        );
        return "<amp-ad width='300' height='250' layout='fixed' type='doubleclick' data-slot='" . $slot . "' data-json='" . json_encode($ampPrebidConfig) . "'></amp-ad>";
    }

    public function insertAMPAds( $content ) {
        if (!$ampAd = $this->getAMPAd()) {
            return $content;
        }
        $closing_p = '</p>';
        $paragraphs = array();
        $skipped_p = 0;
        foreach(explode($closing_p, $content) as $index => $paragraph) {
            array_push($paragraphs, $paragraph);
            if ($skipped_p == 3) {
                array_push($paragraphs, "<p>" . $ampAd);
                $skipped_p = 0;
            } else {
                $skipped_p = $skipped_p + 1;
            }
        }
        return implode($closing_p, $paragraphs);
    }

    public function isAmpPluginInstalled() {
        $allPlugins = get_plugins();
        return $allPlugins['accelerated-mobile-pages/accelerated-moblie-pages.php']['Name'] || $allPlugins['amp/amp.php']['Name'];
    }

    public function isAMPAdsEnabled() {
        $options = get_option( $this->adminSettingsKey );
        return isset( $options['enable_amp_ads'] ) ? ($options['enable_amp_ads'] == 'on') : false;
    }

    public function isAmpPluginActivated() {
        return function_exists('is_plugin_active') && (is_plugin_active( 'accelerated-mobile-pages/accelerated-moblie-pages.php' ) || is_plugin_active( 'amp/amp.php' ));
    }

    public function adminSettingsValidate( $args ){
        if ($args && array_key_exists('enable_amp_ads', $args) && $args['enable_amp_ads'] == 'on') {
            if ( !$this->hasAMPAd() ) {
                $current_user = wp_get_current_user();
                unset($args['enable_amp_ads']);
                $to_email = "support@monumetric.com";
                if(wp_mail($to_email, "AMP request via WordPress plugin (" . get_site_url() . ").", "Hello,\n" . $current_user->user_email . " on " . get_site_url() . " requested to activate AMP on their website.")) {
                    add_settings_error( $this->adminSettingsKey, 'invalid_uuid', "AMP is not yet configured for your account, " . $to_email . " has been notified.", $type = 'error' );
                } else {
                    add_settings_error( $this->adminSettingsKey, 'invalid_uuid', "AMP is not yet configured for your account, please contact your account manager to configure AMP ads for you.", $type = 'error' );
                }
            } else {
                add_settings_error( $this->adminSettingsKey, 'invalid_uuid', "Saved successfully.", $type = 'updated' );
            }
        } else {
            add_settings_error( $this->adminSettingsKey, 'invalid_uuid', "Saved successfully.", $type = 'updated' );
        }
        return $args;
    }

    public function adminInit(){
        register_setting( $this->adminSettingsKey, $this->adminSettingsKey, array( $this, "adminSettingsValidate" ) );
    }

    public function renderManager() {
        if ( !$this->isAmpPluginInstalled() ) {
            echo '<p class="updated notice notice-error" style="padding:11px 15px">Please install one of the AMP plugin from <a href="https://wordpress.org/plugins/accelerated-mobile-pages/">AMP for WP – Accelerated Mobile Pages</a> or <a href="https://wordpress.org/plugins/amp/">AMP</a> to enable ads on AMP pages.</p>';
        } else if ( !$this->isAmpPluginActivated() ) {
            echo '<p class="updated notice notice-error" style="padding:11px 15px">Please activate the AMP plugin to enabled ads on AMP pages.</p>';
        } else {
            ?>
            <form method="post" action="options.php">
                <?php
                settings_fields( $this->adminSettingsKey );
                do_settings_sections( __FILE__ );
                ?>
                <table class="form-table">
                    <tbody>
                        <tr>
                            <th scope="row"><label for="<?php echo $this->adminSettingsKey; ?>[enable_amp_ads]">Enable AMP Compatible Ads</label></th>
                            <td>
                                <input type="checkbox" name="<?php echo $this->adminSettingsKey; ?>[enable_amp_ads]" id="<?php echo $this->adminSettingsKey; ?>[enable_amp_ads]" <?php if($this->isAMPAdsEnabled()) { echo "checked"; } ?> />
                            </td>
                        </tr>
                    </tbody>
                </table>
                <?php submit_button( 'Save Changes', 'primary', 'monu_ampads_save', false ); ?>
            </form>
            <?php
        }
    }
}
