<?php

defined( "ABSPATH" ) or die( "No script kiddies please!" );


class MonumetricReportPosts {

    private $blogUuidCacheKey = 'tbn_ad_settings';
    private $versionReportEndpoint = 'https://imps.monu.delivery/mmt.gif';

    public function __construct(){
        add_filter( 'cron_schedules', array( $this, 'everyFifteenMinutes'));
        add_action( 'mmt_report_posts', array( $this, 'reportPostData') );
    }

    function everyFifteenMinutes( $schedules ) {
        $schedules['mmt_every_fifteen_minutes'] = array(
            'interval'  => 900,
            'display'   => __( 'Every 15 Minutes', 'textdomain' )
        );
        return $schedules;
    }

    /**
     * Log extracted post data to logger;if data already logged for post
     * exclude that post else if post is modified log that also!
     */
    public function reportPostData() {
        global $wpdb;
        global $tablePrefix;
        $blogPostTableNames = array();

        if ( is_multisite() ) {
            $rows = $wpdb->get_results("SELECT blog_id from $wpdb->blogs WHERE public = '1' AND archived = '0' AND mature = '0' AND spam = '0' AND deleted = '0';");
            foreach( $rows as $row ) {
                $blogPostTableNames[$row->blog_id] = $wpdb->get_blog_prefix($row->blog_id);
            }
        } else {
            $rows = array("blog_id" => (string)get_current_blog_id());
            $blogPostTableNames[$rows["blog_id"]] = $wpdb->get_blog_prefix($row->blog_id);
        }
        if ( $rows ) {
            if ( count( $blogPostTableNames ) > 0 ) {
                foreach( $blogPostTableNames as  $blogId => $tablePrefix ) {
                    $mmtOptions = get_option( $this->blogUuidCacheKey );
                    if ( isset($mmtOptions['uuid']) && $mmtOptions['uuid'] != "" ) {
                        if ( is_multisite() ) {
                            switch_to_blog($blogId);
                        }
                        $queryStr = "SELECT ID, $blogId as `blog_id` from (SELECT {$tablePrefix}posts.ID,{$tablePrefix}posts.post_modified_gmt, {$tablePrefix}postmeta.meta_value FROM {$tablePrefix}posts LEFT OUTER JOIN {$tablePrefix}postmeta ON ({$tablePrefix}posts.ID = {$tablePrefix}postmeta.post_id AND {$tablePrefix}postmeta.meta_key='mmtLoggedUntil') WHERE {$tablePrefix}posts.post_status = 'publish') posts where meta_value is null or CONCAT(post_modified_gmt) != meta_value order by post_modified_gmt asc LIMIT 0,10;";

                        $rows = $wpdb->get_results($queryStr);
                        if ( $rows ) {
                            $posts = array();
                            foreach( $rows as $row ) {
                                $postslist[] = is_multisite()?get_blog_post( $row->blog_id, $row->ID ):get_post($row->ID);
                            };
                            if ( !empty($postslist) ) {
                                foreach( $postslist as $post ) {
                                    $categoryData = get_the_category($post->ID);
                                    $categoryNames = array();
                                    foreach( $categoryData as $cd ) {
                                        array_push($categoryNames, $cd->cat_name);
                                    };
                                    $extractedData = array('postId' => $post->ID, 'postTitle' => $post->post_title , 'url' => $post->guid, 'postType' => $post->post_type , 'postCount' => str_word_count($post->post_content), 'imageCount' => substr_count($post->post_content, '<img' ), 'categories' => $categoryNames, 'postStatus' => $post->post_status,'postDate' =>$post->post_date,'author' => get_the_author_meta('display_name', $post->post_author));

                                    $response = wp_remote_get($this->versionReportEndpoint . '?a=b.d&u=' . $mmtOptions['uuid'] .'&d=' . urlencode(json_encode($extractedData))
                                    );

                                    if ( !is_wp_error( $response ) ) {
                                        update_post_meta($post->ID, 'mmtLoggedUntil', $post->post_modified_gmt);
                                    }
                                }
                                $postslist = [];
                            }
                        }
                        if ( is_multisite() ) {
                            restore_current_blog();
                        }
                    }
                }
            }
        }
    }

    public function onActivationOrUpgrade(){
        if ( !wp_next_scheduled( 'mmt_report_posts' ) ) {
            wp_schedule_event( time(), 'mmt_every_fifteen_minutes', 'mmt_report_posts' );
        }
    }

    public function onDeactivation(){
        wp_clear_scheduled_hook('mmt_report_posts');
    }

}
