<?php

defined( "ABSPATH" ) or die( "No script kiddies please!" );

class MonumetricVideo {
    private $vastUrls = null;
    private $videoDetailsUrl = "http://video-info.thebloggernetwork.com/";

    public function __construct( $vastUrls ){
      $this->vastUrls = $vastUrls;
      add_shortcode( "tbn_player", array( $this, "renderShortCode" ) );
      add_action( 'admin_action_tbn_clear_video_poster_cache', array( $this, "clearVideoCache" ) );

    }


    public function clearVideoCache(){
      global $wpdb;
      $wpdb->query("DELETE FROM $wpdb->options WHERE `option_name` LIKE ('tbn_video_details_%')");
      add_settings_error('tbn_ad_settings', 'uuid_invalid_email', 'TEST', $type = 'updated');
      wp_redirect( $_SERVER['HTTP_REFERER'] );
    }

    private function videoDetails( $vid ){
        $details = get_option( 'tbn_video_details_' .  $vid );
        if( $details === false ){
            $response = wp_remote_get( $this->videoDetailsUrl . $vid . ".json" );
            if( !is_wp_error($response) && $response['response']['code'] == 200 ) {
                $details = $response['body'];
                update_option( 'tbn_video_details_' . $vid, $details);
            }
        }
        if( $details ){
            $details = json_decode( $details );
        }
        return $details;
    }

    public function videoHtml( $vid, $details, $quality, $showAds, $width){
        $video = isset( $details->videos->$quality ) ? $details->videos->$quality : null;
        $video = ( !$video && isset( $details->videos->sd ) ) ? $details->videos->sd : $video;
        if( !$video ){
            return "";
        }
        $thumb = $details->thumbnail ? ( "config['image'] = '" . $details->thumbnail->link . "';" ) : null;
        $vastConfig = "";
        if( $this->vastUrls && $showAds ){
            $vastConfig = "config['advertising'] = {client: \"vast\", tag: " . json_encode( $this->vastUrls ) . "};";
        }
        return <<<HEREDOCS
        <div style="max-width:$width"><div style="position:relative; width:100%;" id="wrap_tbn_video_id_$vid"><div class="tbn_video_player"><div id="tbn_video_id_$vid"></div></div></div></div>
        <script data-cfasync="false" type="text/javascript">
            window.tbnVideoFns = window.tbnVideoFns || []
            tbnVideoFns.push(function(){
                var width = document.getElementById("wrap_tbn_video_id_$vid").offsetWidth;
                var config = {
                    primary:"flash",
                    file: "$video->link_secure",
                    stretching: "exactfit",
                    width: width,
                    height: width * ($video->height/$video->width)
                };
                $thumb
                $vastConfig
                jwplayer("tbn_video_id_$vid").setup(config);
        });
        </script>
HEREDOCS;
    }

    public function renderShortCode( $atts ){
        $options = shortcode_atts( array(
            'id' => null,
            'ads' => 'yes',
            'quality' => 'hd',
            'width' => '100%'
        ), $atts );
        if ( !$options['id'] || $options['id'] == "" ){
            return "";
        }
        $options['ads'] = (trim(strtolower( (string)$options['ads'])));
        $options['ads'] = !($options['ads'] == "false" || $options['ads'] == "no");
        $options["quality"] = strtolower( $options["quality"] );
        $options['quality'] = ( $options['quality'] == "hd" || $options['quality'] == "sd" ) ? $options['quality'] : 'hd';
        $options["width"] = strtolower( trim ( $options["width"] ) );
        if( is_numeric( $options["width"] ) ){
          $options["width"] = $options["width"] . "px";
        }
        if( !( ( substr($options["width"], -1) == "%" && is_numeric(substr($options["width"], 0, -1)) && ((int) substr($options["width"], 0, -1))<=100 ) ||  ( substr($options["width"], -2) == "px" && is_numeric(substr($options["width"], 0, -2)) ) ) ){
          $options["width"] = "100%";
        }
        $data = $this->videoDetails( $options['id'] );
        if ( !$data ) {
            return "";
        }
        return $this->videoHtml( $options['id'], $data, $options['quality'], $options['ads'], $options['width'] );
    }

    public function renderManager(){
        require( dirname( __FILE__ ) . "/video-manager.php" );
    }
}
