<?php
/**
 * Monumetric Ads
 *
 * @package    MonumetricAds
 * @author     Aman Kumar Jain
 * @authoruri  http://www.amanjain.com/
 * @copyright  Copyright (c) 2014, The Blogger Network, LLC.
 * @license    GPLv2 or later
 * @licenseuri http://www.gnu.org/licenses/gpl-2.0.html
 *
 * @wordpress-plugin
 * Plugin Name: Monumetric Ads
 * Description: A plugin for implementation of various ads from Monumetric. NOTE: Deactivating this plugin will disable ads on your site.
 * Version: 7.4.3
 * Author: Monumetric
 * Author URI: http://www.monumetric.com
 * Requires PHP: 5.3
 * License: GPLv2
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */



defined( "ABSPATH" ) or die( "No script kiddies please!" );

$basePath = dirname( __FILE__ );
require( $basePath . "/libs/advertisement.php" );
// require( $basePath . "/libs/amp-ads.php" );
require( $basePath . "/libs/report-posts.php" );
require( $basePath . "/libs/AdminNotices/mmt-notifications.php" );

class MonumetricAds {
    private $version = '7.4.3';
    private $menuOptionSlug = 'monumetric-manager';
    private $versionReportEndpoint = 'http://imps.monu.delivery/mmt.gif';
    private $tbnAdvertisement = null;
    // private $ampAds = null;
    private $monuAdsTxt = null;
    private $monuReportPosts = null;
    private $notifications = null;
    private $pluginVersionCacheKey = "monumetric_ads_plugin_version";
    private $blogUuidCacheKey = 'tbn_ad_settings';
    private $adminSettingsKey = "monu_adstxt_config";

    public function __construct(){
        $this->monuAdsTxt = new MonumetricAdsTxt();
        // $this->ampAds = new AMPAds();
        $this->tbnAdvertisement = new MonumetricAdvertisement();
        $adDetails = $this->tbnAdvertisement->getAdDetails();
        $this->monuReportPosts = new MonumetricReportPosts();
        $this->notifications = new MonumetricNotifications();

        add_action( 'plugins_loaded', array( $this, 'pluginsLoaded') );
        add_action( 'admin_menu', array( $this, 'setManagerMenu' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueueScript' ) );
        add_action( 'tbn_report_plugin_version', array( $this, 'reportVersion') );
        add_action( 'tbn_report_exclusions', array( $this, 'reportExclusions') );
        add_filter( 'plugin_action_links_' . plugin_basename(__FILE__),  array( $this, 'pluginActions' ));
        add_action( 'init', array( $this, 'initializeRedirect') );

        register_deactivation_hook( __FILE__, array( $this, 'onDeactivation') );
        register_activation_hook( __FILE__, array( $this, 'onActivationOrUpgrade') );
    }

    public function enqueueScript(){
        wp_enqueue_style( 'tbn-main-styles', plugins_url( 'style/styles.css', __FILE__ ), array(), '0.3', 'all' );
    }

    public function reportVersion() {
        $mmtOptions = get_option( $this->blogUuidCacheKey );
        $pluginVersion = get_site_option( $this->pluginVersionCacheKey );

        if (!isset($mmtOptions['uuid'])) { //  || $pluginVersion == $this->version
            return false;
        }
        global $wp_version;
        $response = wp_remote_get(
            $this->versionReportEndpoint . '?a=w.v&u=' . $mmtOptions['uuid'] .
            '&d=' . urlencode( json_encode( array(
                'pluginVersion' => $this->version, 'wordpressVersion' => $wp_version
            ) ) )
        );
        if ( !is_wp_error( $response ) ) {
            update_option( $this->pluginVersionCacheKey, $this->version );
        }
    }

    public function reportExclusions() {
        $mmtOptions = get_option( $this->blogUuidCacheKey );
        if(!isset($mmtOptions['exclusions']['user-roles'])) {
          return false;
        }
        $response = wp_remote_get(
            $this->versionReportEndpoint . '?a=e.a&u=' . $mmtOptions['uuid'] .
            '&d=' . urlencode( json_encode( array(
                'exclusions' => json_encode($mmtOptions['exclusions']['user-roles'])
              ) ) )
        );
    }

    /**
     * Checks redirect status and if applicable will initialize the redirect and verify that 
     * it is working properly.
     */
    public function initializeRedirect() {
        $options = get_option($this->adminSettingsKey);
        $redirectEnabled = isset($options['enable_redirect']) && ($options['enable_redirect'] !== false);
        $confirmedRedirect = isset($options['confirm_redirect']) && ($options['confirm_redirect'] !== false);
        if($options) {
            if ( $redirectEnabled ) {
                $redirect = $this->monuAdsTxt->addRedirect();
                if (!$confirmedRedirect && $redirect) {
                    $confirmation = $this->monuAdsTxt->verifyAdstxtRedirect();
                }
                return;
            } else {
                if ($confirmedRedirect) {
                    $this->monuAdsTxt->resetRedirectSettings();
                }
            }
        }
        $this->initializeNotifications();
    }

    public function initializeNotifications() {
      $this->notifications->init();
    }

    public function renderManager(){
        $tabs = array(
            'default' => array( 'label' => 'Default Settings', 'obj' => $this->tbnAdvertisement ),
            'adstxt' => array( 'label' => 'Ads.txt', 'obj' => $this->monuAdsTxt ),
            // 'ampads' => array( 'label' => 'AMP Compatible Ads', 'obj' => $this->ampAds )
        );
        $currentTab = isset($tabs[filter_input(INPUT_GET, 'tab')]) ? filter_input(INPUT_GET, 'tab') : "default";
        $manager = null;
        ?>

        <div class="wrap monu-wrap">
            <div class="monu-header">
                <div class="logo">
                    <img src="<?php echo plugins_url( 'monumetric-logo.png', __FILE__ ); ?>" alt="Monumetric" />
                </div>
                <div class="outer-link">
                    <a class="monu-button" href="https://console.monumetric.com/auth/login" target="_BLANK">
                         Monumetric Console <span class="dashicons dashicons-external"></span>
                    </a>
                </div>
                <div class="clearfix"></div>
            </div>
            <div class="monu-navigation">
                <div class="nav-tab-wrapper">
                    <?php
                    foreach( $tabs as $tab => $details ){
                        $class = "";
                        if( $tab == $currentTab ){
                            $class =  ' nav-tab-active';
                            $manager = $details['obj'];
                        }
                        echo "<a class='nav-tab$class' href='?page=monumetric-manager&tab=$tab'>" . $details['label'] . "</a>";
                    }
                    ?>
                </div>
            </div>
            <?php
            if( $manager ){
                $manager->renderManager();
            }
            ?>
        </div>
    <?php
    }

    public function setManagerMenu(){
        add_options_page( 'Monumetric Settings', 'Monumetric', 'manage_options', $this->menuOptionSlug, array( $this, 'renderManager' ) );
    }

    public function pluginsLoaded() {
        $this->monuAdsTxt->pluginsLoaded();
    }

    public function onActivationOrUpgrade(){
        if ( !wp_next_scheduled( 'tbn_report_plugin_version' ) ) {
            wp_schedule_event( time(), 'twicedaily', 'tbn_report_plugin_version' );
        }
        if ( !wp_next_scheduled( 'tbn_report_exclusions' ) ) {
            wp_schedule_event( time(), 'twicedaily', 'tbn_report_exclusions' );
        }
        $this->tbnAdvertisement->onActivationOrUpgrade();
        $this->monuAdsTxt->onActivationOrUpgrade();
        $this->monuReportPosts->onActivationOrUpgrade();
    }

    public function onDeactivation(){
        wp_clear_scheduled_hook( 'tbn_report_plugin_version' );
        wp_clear_scheduled_hook( 'tbn_report_exclusions' );
        $this->tbnAdvertisement->onDeactivation();
        $this->monuAdsTxt->onDeactivation();
        $this->monuReportPosts->onDeactivation();
    }

    public function pluginActions( $links ){
        return array_merge( array('<a href="'. get_admin_url(null, 'options-general.php?page=' . $this->menuOptionSlug ) .'">Settings</a>'), $links);
    }

}

$monumetricAds = new MonumetricAds();


//Add custom WP Update Checker
require $basePath . '/libs/puc/1.6/plugin-update-checker.php';
$MyUpdateChecker = PucFactory::buildUpdateChecker(
    'http://monu.delivery/wp-plugins/monumetric-ads/version.json',
    __FILE__,
    'monumetric-ads'
);
